# -*- coding: utf-8 -*-

from django_filters import rest_framework as djfilters
from django.utils import timezone
from clariweb.clariweb.models import DocumentPerUser


class ClinicFilter(djfilters.FilterSet):
    clinic_ipp = djfilters.CharFilter(field_name="document__clinic_ipp", lookup_expr='icontains')
    clinic_hospi = djfilters.CharFilter(field_name="document__clinic_hospi", lookup_expr='icontains')
    lastname = djfilters.CharFilter(field_name="document__patient__last_name", lookup_expr='istartswith')
    firstname = djfilters.CharFilter(field_name="document__patient__first_name", lookup_expr='istartswith')
    clinic_service = djfilters.CharFilter(field_name="document__user__service_clinic", lookup_expr='icontains')
    date = djfilters.CharFilter(field_name='document__date', method='datefilter')

    def datefilter(self, queryset, name, value):
        now = timezone.now()
        if value not in ("0", "1", "3", "7", "30"):
            raise ValueError("only 0, 1, 3, 7 or 30 days are supported")
        if value == '0':
            return queryset
        else:
            dt = now - timezone.timedelta(days=int(value))
        return queryset.filter(
            document__sil_demande_date__gte=timezone.datetime(
                dt.year, dt.month, dt.day, tzinfo=dt.tzinfo))

    class Meta:
        model = DocumentPerUser
        fields = [
            'document__patient__last_name', 'document__patient__first_name',
            'document__clinic_ipp', 'document__clinic_hospi', 'document__user__service_clinic'
        ]

