# -*- coding: utf-8 -*-

import sys
from collections import OrderedDict
from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response


class StandardPagination(PageNumberPagination):
    page_size = 50
    max_page_size = 100
    page_size_query_param = None


class NoCountPagination(StandardPagination):
    def count(self):
        return sys.maxsize

    def get_paginated_response(self, data):
        return Response(OrderedDict([
            ('page', self.page.number),
            ('pagesize', self.page_size),
            ('results', data)
        ]))


class LoginAsUserPagination(NoCountPagination):
    page_size = 10
    max_page_size = 10


class DocumentListPagination(StandardPagination):
    def get_paginated_response(self, data):
        """Just remove the "count" of the result list (costs performance)"""
        return Response(OrderedDict([
            ('next', self.get_next_link()),
            ('previous', self.get_previous_link()),
            ('results', data)
        ]))
