# -*- coding: utf-8 -*-

import six
from rest_framework.authentication import get_authorization_header
from rest_framework.permissions import BasePermission
from django.conf import settings


class TokenPermission(BasePermission):
    """
    Simple token based authentication.

    Clients should authenticate by passing the token key in the "Authorization"
    HTTP header, prepended with the string "Token ".  For example:

    Authorization: Token 401f7ac837da42b97f613d789819ff93537bee6a
    """

    def has_permission(self, request, view):
        auth = get_authorization_header(request).split()
        if not auth or auth[0].lower() != b'token':
            return False
        if len(auth) == 1:
            return False
        elif len(auth) > 2:
            return False
        key = auth[1]
        if isinstance(settings.API_RPC_TOKEN, six.string_types):
            settings_token = settings.API_RPC_TOKEN.encode("ascii")
        else:
            settings_token = settings.API_RPC_TOKEN
        return (key == settings_token)
