# -*- coding: utf-8 -*-

from rest_framework import serializers
from clariweb.clariweb.models import (
    Actor,
    Document,
    BasicUser,
    ClariwebUser
)


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = BasicUser
        fields = ('username', 'first_name', 'last_name', 'email', 'is_staff',
                  'is_confirmed', 'is_active', 'localpassword')

    is_confirmed = serializers.BooleanField(read_only=True)
    is_active = serializers.BooleanField(read_only=True)
    localpassword = serializers.BooleanField(read_only=True)


class RestrictedUserSerializer(serializers.ModelSerializer):
    class Meta:
        model = BasicUser
        fields = ('first_name', 'last_name')


class RestrictedClariwebUserSerializer(serializers.ModelSerializer):
    class Meta:
        model = ClariwebUser
        fields = ('user', 'civilite', 'service_clinic')

    user = RestrictedUserSerializer()

    def to_representation(self, obj):
        ret = super().to_representation(obj)
        user = self.context["request"].user
        if user.clariwebuser.kind == "PAT":
            ret["service_clinic"] = []
        return ret


class ActorSerializer(serializers.ModelSerializer):
    class Meta:
        model = Actor
        fields = ('first_name', 'last_name', 'civilite', 'date_naissance')


class FullClariwebUserSerializer(serializers.ModelSerializer):
    class Meta:
        model = ClariwebUser
        fields = ('user', 'kind', 'civilite', 'date_naissance',
                  'address_city', 'address_country', 'address_postal_code',
                  'address_line1', 'address_line2', 'address_line3',
                  'address_phone', 'address_mobile_phone',
                  'group', 'service_clinic', 'debpt')

    user = UserSerializer()


class DocumentSerializer(serializers.ModelSerializer):
    class Meta:
        model = Document
        fields = ('id',
                  'sil_demande_number', 'sil_demande_date',
                  'date_add', 'content_type',
                  'document', 'patient', 'medecin', 'correspondant',
                  'user', 'is_invoice', 'analyses', 'unpaid_amount',
                  'datafile',)

    user = RestrictedClariwebUserSerializer(required=False)
    patient = ActorSerializer(required=False)
    medecin = ActorSerializer(required=False)
    correspondant = ActorSerializer(required=False)
    analyses = serializers.ListField()
    datafile = serializers.SerializerMethodField()

    def get_datafile(self, obj):
        return bool(obj.datafile)

    def to_representation(self, obj):
        ret = super().to_representation(obj)
        user = self.context["request"].user
        if user.clariwebuser.kind == "PAT":
            ret["analyses"] = []
        return ret

class LoginSerializer(serializers.Serializer):
    username = serializers.CharField(max_length=128)
    password = serializers.CharField(max_length=128)


class LoginInvoiceSerializer(serializers.Serializer):
    invoice_id = serializers.CharField(max_length=32)
    birthdate = serializers.DateField()
