from django.conf.urls import url
from rest_framework.urlpatterns import format_suffix_patterns
from clariweb.api import views as api_views
from clariweb.api.clinic import views as api_clinic_views
from clariweb.api.admin import views as api_admin_views
from clariweb.api.payment import views as api_payment_views
from clariweb.api.passwordreset import views as passwordreset_views


urlpatterns = format_suffix_patterns([
    url(
        r'^login$',
        api_views.LoginView.as_view(),
        name='api-login'
    ),
    url(
        r'^login/setsessionexpiry$',
        api_views.SetSessionExpiryView.as_view(),
        name='api-login-setsessionexpiry'
    ),
    url(
        r'^login/byinvoice$',
        api_views.LoginInvoiceView.as_view(),
        name='api-login-invoice',
    ),
    url(
        r'^logout$',
        api_views.LogoutView.as_view(),
        name='api-logout'
    ),
    url(
        r'^emailconfirm$',
        api_views.EmailConfirmView.as_view(),
        name='api-emailconfirm'
    ),
    url(
        r'^emailconfirm/resend$',
        api_views.EmailConfirmResendView.as_view(),
        name="api-resendemailconfirm"
    ),
    url(
        r'^passwordreset/ask$',
        passwordreset_views.AskView.as_view(),
        name="api-passwordreset-ask"
    ),
    url(
        r'^passwordreset/check$',
        passwordreset_views.CheckView.as_view(),
        name="api-passwordreset-check"
    ),
    url(
        r'^passwordreset/edit$',
        passwordreset_views.EditView.as_view(),
        name="api-passwordreset-edit"
    ),
    url(
        r'^document/listall$',
        api_views.ListDocumentView.as_view(),
        name="document-listall"
    ),
    url(
        r'^document/listnotseen$',
        api_views.ListNotSeenDocumentView.as_view(),
        name="document-listnotseen"
    ),
    url(
        r'^document/(?P<pk>\d+)/detail$',
        api_views.DocumentDetailView.as_view({
            'get': 'retrieve'
        }),
        name="document-detail"
    ),
    url(
        r'document/clinic/list$',
        api_clinic_views.ListDocumentClinicView.as_view(),
        name='document-clinic-list'
    ),
    url(
        r'document/clinic/list/noshareservice$',
        api_clinic_views.ListDocumentClinicNoShareServicesView.as_view(),
        name='document-clinic-list-noshareservice'
    ),
    url(
        r'^document/clinic/(?P<pk>\d+)/detail$',
        api_clinic_views.DocumentClinicDetailView.as_view({
            'get': 'retrieve'
        }),
        name="document-clinic-detail"
    ),
    url(
        r'^payment/request$',
        api_payment_views.PaymentRequestView.as_view(),
        name='api-payment-request'
    ),
    url(
        r'^payment/request/list$',
        api_payment_views.PaymentRequestListView.as_view(),
        name='api-payment-request-list'
    ),
    url(
        r'^payment/response$',
        api_payment_views.PaymentResponseView.as_view(),
        name='api-payment-response'
    ),
    url(
        r'^settings$',
        api_views.CustomSettingsView.as_view(),
        name='api-settings'
    )
])

urlpatterns += format_suffix_patterns([
    url(
        r'^admin/notification$',
        api_admin_views.NotificationFilteringView.as_view(),
        name='api-admin-notification'
    ),
    url(
        r'^admin/accesslog$',
        api_admin_views.AccessLogFilteringView.as_view(),
        name='api-admin-accesslog'
    ),
    url(
        r'admin/smscount$',
        api_admin_views.SMSCountView.as_view({
            'get': 'get_sms_sent'
        }),
        name='api-admin-smscount'
    ),
    url(
        r'admin/smscount/quota$',
        api_admin_views.SMSCountView.as_view({
            'get': 'get_sms_quota'
        }),
        name='api-admin-smscount-getquota'
    ),
    url(
        r'admin/smscount/quota/update$',
        api_admin_views.SMSCountView.as_view({
            'get': 'update_sms_quota'
        }),
        name='api-admin-smscount-updatequota'
    ),
    url(
        r'admin/smscount/buy$',
        api_admin_views.SMSBuyView.as_view(),
        name='api-admin-smscount-buy'
    ),
    url(
        r'admin/loginas/selector$',
        api_admin_views.LoginAsUserSelector.as_view({
            'get': 'list'
        }),
        name='api-admin-loginas-selector'
    ),
    url(
        r'admin/loginas$',
        api_admin_views.LoginAsUserView.as_view(),
        name='api-admin-loginas'
    ),
    url(
        r'admin/usermodify$',
        api_admin_views.UserModifyView.as_view(),
        name='api-admin-user-modify'
    ),
    url(
        r'admin/settings$',
        api_admin_views.CustomSettingsView.as_view(),
        name='api-admin-settings'
    ),
    url(
        r'admin/stats/month$',
        api_admin_views.DocumentStatisticsPerMonthGraphView.as_view(),
        name='api-admin-stats-document-month-graph'
    ),
    url(
        r'admin/stats/month/user$',
        api_admin_views.DocumentStatisticsPerMonthPerUserView.as_view(),
        name='api-admin-stats-document-month-user'
    ),
    url(
        r'admin/payments$',
        api_admin_views.PaymentResponseFilteringView.as_view(),
        name='api-admin-payments'
    ),
])


urlpatterns += [
    url(
        r"^document/(?P<document_pk>\d+)/download/?(?P<filename>.+\.pdf)*$",
        api_views.DocumentDownload.as_view(),
        name="document-download"
    ),
    url(
        r"^document/(?P<document_pk>\d+)/downloadstream/$",
        api_views.DocumentDownloadStream.as_view(),
        name="document-downloadstream"
    ),
]
