# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals

import ipaddress
import constance
import logging
from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend
from django.db.models import Q

logger = logging.getLogger(__name__)


def _is_ip_in(ip, strnetlist):
    if hasattr(ip, 'ipv4_mapped') and ip.ipv4_mapped is not None:
        ip = ip.ipv4_mapped
    if settings.DEBUG:
        strnetlist.extend(['::1/128', '127.0.0.0/8'])
    for strn in strnetlist:
        try:
            if ip in ipaddress.ip_network("{}".format(strn)):
                return True
        except ValueError:
            # Invalid network address in the configuration file
            pass
    logger.warning("denied access from %s (not in %s whitelist)",
        ip, strnetlist)
    return False


class ClariwebAuthBackend(ModelBackend):
    def authenticate(self, request, **kwargs):
        user = self._authenticate(username=kwargs.pop("username", None),
                                  password=kwargs.pop("password", None),
                                  birthdate=kwargs.pop("birthdate", None),
                                  **kwargs)
        if not request:
            return None
        if user:
            ip = ipaddress.ip_address("{}".format(request.META["REMOTE_ADDR"]))
            if not self.pass_ipprotect(user, ip):
                return None
        return user

    def pass_ipprotect(self, user, ip):
        if user.is_anonymous:
            return True
        if user.is_staff or user.is_superuser:
            return _is_ip_in(ip, constance.config.SECURITY_IP_ADDRESSES_ADMIN)
        else:
            clariwebuser = user.clariwebuser
            if clariwebuser:
                if clariwebuser.kind == "COR" and clariwebuser.group_clinic:
                    return _is_ip_in(
                        ip, constance.config.SECURITY_IP_ADDRESSES_CLINIC)
        return True

    def _authenticate(self, username=None, password=None, birthdate=None,
                      **kwargs):
        usermodel = get_user_model()
        if birthdate is not None:
            return usermodel._default_manager.get(
                username=username, clariwebuser__date_naissance=birthdate)
        else:
            queryset = usermodel._default_manager.filter(
                Q(username__iunaccent__iexact=username) |
                Q(email__iunaccent__iexact=username)
            )
        if queryset.count() > 0:
            for user in queryset:
                if user.check_password(password):
                    return user
        else:
            # Run the default password hasher once to reduce the timing
            # difference between an existing and a non-existing user (#20760).
            usermodel().set_password(password)
