# -*-  coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from importlib import import_module


def SessionMiddleware(get_response):
    SESSION_HEADER_NAME = "HTTP_X_SESSION_ID"
    SESSION_GET_NAME = "sessionid"

    def middleware(request):
        engine = import_module(settings.SESSION_ENGINE)
        session_key = request.META.get(SESSION_HEADER_NAME, None)
        if not session_key:
            # Try using request.GET parameters
            session_key = request.GET.get(SESSION_GET_NAME, None)
        request.session = engine.SessionStore(session_key)
        # Reset the session expiry for users logged with very short sessions
        if request.session.get_expiry_age() == settings.SESSION_COOKIE_AGE:
            request.session.set_expiry(settings.SESSION_COOKIE_AGE)
        response = get_response(request)
        if response.status_code != 500:
            request.session.save()
        return response
    return middleware


def NoIECacheMiddleware(get_response):
    IE_UA = """Mozilla/4.0 (compatible; MSIE"""

    def middleware(request):
        response = get_response(request)
        ua = request.META.get("HTTP_USER_AGENT", "")
        if ua.startswith(IE_UA):
            response["Cache-Control"] = "no-cache, no-store"
            response["Expires"] = "-1"
            response["Pragma"] = "no-cache"
        return response
    return middleware


def RemoteAddrMiddleware(get_response):
    def middleware(request):
        try:
            real_ip = request.META['HTTP_X_FORWARDED_FOR']
        except KeyError:
            pass
        else:
            # HTTP_X_FORWARDED_FOR can be a comma-separated list of IPs. The
            # client's IP will be the first one.
            real_ip = real_ip.split(",")[0].strip()
            request.META['REMOTE_ADDR'] = real_ip
        return get_response(request)
    return middleware
