# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib.postgres.operations import UnaccentExtension
from django.db import models, migrations
import re
import django.utils.timezone
import django.core.files.storage
import django.db.models.deletion
from django.conf import settings
import clariweb.clariweb.models
import phonenumber_field.modelfields
import django.core.validators
import simple_email_confirmation.models


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0006_require_contenttypes_0002'),
    ]

    operations = [
        UnaccentExtension(),
        migrations.CreateModel(
            name='DocumentStatisticsPerMonth',
            fields=[
                ('timestep', models.DateField(serialize=False, primary_key=True)),
                ('documents', models.PositiveIntegerField(null=True, blank=True)),
            ],
            options={
                'ordering': ['-timestep'],
                'db_table': 'clariweb_stats_documents_per_month',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='BasicUser',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(null=True, verbose_name='last login', blank=True)),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('username', models.CharField(help_text='Required. 128 characters or fewer. Letters, numbers and @/./+/-/_ characters', unique=True, max_length=128, verbose_name='username', validators=[django.core.validators.RegexValidator(re.compile(b'^[\\w.@+-]+$'), 'Enter a valid username.', b'invalid')])),
                ('email', models.EmailField(db_index=True, max_length=256, verbose_name='email address', blank=True)),
                ('first_name', models.CharField(max_length=256, verbose_name='first name', blank=True)),
                ('last_name', models.CharField(max_length=256, verbose_name='last name', blank=True)),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('localpassword', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name': 'utilisateur',
            },
            bases=(simple_email_confirmation.models.SimpleEmailConfirmationUserMixin, models.Model),
        ),
        migrations.CreateModel(
            name='AccessLog',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('username', models.CharField(max_length=255)),
                ('date', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('message', models.CharField(max_length=32, choices=[(b'AUTH_OK', 'Authentification r\xe9ussie'), (b'AUTH_BAD', 'Authentification \xe9chou\xe9e')])),
                ('ip', models.GenericIPAddressField(null=True, unpack_ipv4=True, blank=True)),
                ('user', models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.DO_NOTHING, blank=True, to=settings.AUTH_USER_MODEL, null=True)),
            ],
            options={
                'ordering': ['-date'],
            },
        ),
        migrations.CreateModel(
            name='Actor',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('last_name', models.CharField(max_length=256, verbose_name='last name', blank=True)),
                ('first_name', models.CharField(max_length=256, verbose_name='first name', blank=True)),
                ('civilite', models.CharField(max_length=128, blank=True)),
                ('date_naissance', models.DateField(null=True, blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='ClariwebUser',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('sil_pk', models.BigIntegerField(unique=True, verbose_name='id_sil', db_index=True)),
                ('kind', models.CharField(max_length=32, choices=[(b'MED', 'M\xe9decin'), (b'PAT', 'Patient'), (b'COR', 'Correspondant'), (b'PREL', 'Pr\xe9leveur')])),
                ('group', models.CharField(db_index=True, max_length=1024, blank=True)),
                ('civilite', models.CharField(max_length=128, blank=True)),
                ('date_naissance', models.DateField(null=True, blank=True)),
                ('numero_secu', models.CharField(db_index=True, max_length=32, verbose_name='num\xe9ro de s\xe9curit\xe9 sociale', blank=True)),
                ('address_city', models.CharField(max_length=256, blank=True)),
                ('address_country', models.CharField(max_length=128, blank=True)),
                ('address_postal_code', models.CharField(max_length=16, blank=True)),
                ('address_line1', models.CharField(max_length=256, blank=True)),
                ('address_line2', models.CharField(max_length=256, blank=True)),
                ('address_line3', models.CharField(max_length=256, blank=True)),
                ('address_phone', phonenumber_field.modelfields.PhoneNumberField(max_length=128, null=True, blank=True)),
                ('address_mobile_phone', phonenumber_field.modelfields.PhoneNumberField(max_length=128, null=True, blank=True)),
                ('date_creation', models.DateField(auto_now_add=True)),
                ('debpt', models.PositiveIntegerField(default=0, verbose_name="D\xeatte en centimes d'euros")),
            ],
        ),
        migrations.CreateModel(
            name='Document',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('sil_document_pk', models.BigIntegerField(unique=True)),
                ('sil_demande_pk', models.BigIntegerField()),
                ('sil_demande_number', models.CharField(max_length=128)),
                ('sil_demande_date', models.DateTimeField(db_index=True)),
                ('date_add', models.DateTimeField(auto_now_add=True)),
                ('datafile', models.FileField(storage=django.core.files.storage.FileSystemStorage(location=settings.DOCUMENTS_ROOT), upload_to=clariweb.clariweb.models.get_document_upload_path, db_index=True)),
                ('content_type', models.CharField(max_length=128, choices=[(b'PDF', b'application/pdf'), (b'JPG', b'image/jpeg')])),
                ('document', models.CharField(max_length=128)),
                ('is_invoice', models.BooleanField(default=False, verbose_name='Estce une facture')),
                ('available', models.BooleanField(default=True)),
                ('notify_sms', models.BooleanField(default=False)),
                ('notify_email', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name': 'document',
            },
        ),
        migrations.CreateModel(
            name='DocumentAccessLog',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('date', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('ip', models.GenericIPAddressField(null=True, unpack_ipv4=True, blank=True)),
                ('document', models.ForeignKey(related_name='+', to='clariweb.Document', on_delete=models.CASCADE)),
                ('user', models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.DO_NOTHING, to='clariweb.ClariwebUser')),
            ],
            options={
                'ordering': ['-date'],
            },
        ),
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('status', models.CharField(max_length=32, choices=[(b'WAITING', 'en attente'), (b'SENT', 'envoy\xe9'), (b'CONFIRMED', 'envoi confirm\xe9'), (b'ERROR', 'en erreur')])),
                ('error', models.TextField(blank=True)),
                ('date', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('sms', models.CharField(max_length=32, blank=True)),
                ('email', models.EmailField(max_length=256, blank=True)),
                ('origin', models.TextField(blank=True)),
                ('user', models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.DO_NOTHING, to='clariweb.ClariwebUser')),
            ],
        ),
        migrations.CreateModel(
            name='PasswordReset',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('valid_until', models.DateTimeField()),
                ('validation_code', models.CharField(max_length=16)),
                ('user', models.OneToOneField(related_name='+', on_delete=django.db.models.deletion.DO_NOTHING, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='document',
            name='access_log',
            field=models.ManyToManyField(related_name='al+', through='clariweb.DocumentAccessLog', to='clariweb.ClariwebUser'),
        ),
        migrations.AddField(
            model_name='document',
            name='correspondant',
            field=models.ForeignKey(related_name='documents_correspondant', on_delete=django.db.models.deletion.DO_NOTHING, blank=True, to='clariweb.Actor', null=True),
        ),
        migrations.AddField(
            model_name='document',
            name='medecin',
            field=models.ForeignKey(related_name='documents_medecin', on_delete=django.db.models.deletion.DO_NOTHING, blank=True, to='clariweb.Actor', null=True),
        ),
        migrations.AddField(
            model_name='document',
            name='patient',
            field=models.ForeignKey(related_name='documents_patient', on_delete=django.db.models.deletion.DO_NOTHING, to='clariweb.Actor'),
        ),
        migrations.AddField(
            model_name='document',
            name='user',
            field=models.ForeignKey(related_name='documents_user', on_delete=django.db.models.deletion.DO_NOTHING, to='clariweb.ClariwebUser'),
        ),
        migrations.AddField(
            model_name='basicuser',
            name='clariwebuser',
            field=models.OneToOneField(related_name='user', null=True, blank=True, to='clariweb.ClariwebUser', on_delete=django.db.models.deletion.CASCADE),
        ),
        migrations.AddField(
            model_name='basicuser',
            name='groups',
            field=models.ManyToManyField(related_query_name='user', related_name='user_set', to='auth.Group', blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', verbose_name='groups'),
        ),
        migrations.AddField(
            model_name='basicuser',
            name='user_permissions',
            field=models.ManyToManyField(related_query_name='user', related_name='user_set', to='auth.Permission', blank=True, help_text='Specific permissions for this user.', verbose_name='user permissions'),
        ),
        migrations.AlterUniqueTogether(
            name='document',
            unique_together=set([('sil_demande_pk', 'sil_demande_date', 'user', 'document')]),
        ),
        migrations.RunSQL(
            """
        CREATE MATERIALIZED VIEW clariweb_stats_documents_per_month AS SELECT
            timesteps.timestep AS timestep, d.user_id, d.count AS documents
        FROM
            (SELECT
                date_trunc('month', date_add) AS timestep,
                user_id,
                count(id) AS count
             FROM
                clariweb_document
             GROUP BY
                timestep,
                user_id
            ) AS d
        RIGHT OUTER JOIN
            (SELECT
                date_trunc('month', timestep) as timestep
             FROM
                generate_series(now() - '1 year'::interval,
                                now(),
                                '1 month'::interval) AS timestep
             ) AS timesteps
          ON d.timestep = timesteps.timestep
        ORDER BY timestep;
        CREATE INDEX ON clariweb_stats_documents_per_month(user_id);
        ALTER FUNCTION unaccent(text) IMMUTABLE;
        CREATE INDEX clariweb_basicuser_unaccent_idx ON clariweb_basicuser (unaccent(first_name), unaccent(last_name));
            """
        ),
    ]
