# Generated by Django 2.1 on 2018-09-05 15:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clariweb', '0031_add_document_peruser'),
    ]

    operations = [
        migrations.RunSQL("""
CREATE OR REPLACE FUNCTION update_document_per_user(_document_id integer) RETURNS VOID AS $$
    DECLARE
        _user record;
        _document record;
        _user_ids integer[];
        _temp record;
    BEGIN
        SELECT INTO _document * FROM clariweb_document WHERE id = _document_id;
        SELECT INTO _user * FROM clariweb_clariwebuser WHERE id = _document.user_id;
        _user_ids := _user_ids || _document.user_id;
        IF _user.kind = 'COR' OR _user.kind = 'MED' THEN
            /* Transmetteurs */
            IF _user.group IS NOT NULL AND _user.group != '' THEN
                SELECT INTO _temp array_agg(id) AS ids
                    FROM clariweb_clariwebuser
                    WHERE kind in ('COR', 'MED') AND "group" = _user.group;
                _user_ids := _user_ids || _temp.ids;
            END IF;
            /* Cliniques */
            IF _user.kind = 'COR' AND _user.group_clinic IS NOT NULL AND _user.group_clinic != '' THEN
                SELECT INTO _temp array_agg(id) AS ids
                    FROM clariweb_clariwebuser
                    WHERE kind = 'COR' AND group_clinic = _user.group_clinic;
                _user_ids := _user_ids || _temp.ids;
            END IF;
        /* Preleveurs */
        ELSIF _user.kind = 'PREL' THEN
            IF _user.group IS NOT NULL AND _user.group != '' THEN
                SELECT INTO _temp array_agg(id) AS ids
                    FROM clariweb_clariwebuser
                    WHERE kind in ('PREL') AND "group" = _user.group;
                _user_ids := _user_ids || _temp.ids;
            END IF;
        END IF;
        /* Supprime tout acces n'etant plus d'actualite */
        DELETE FROM clariweb_documentperuser WHERE document_id = _document_id AND NOT (user_id = ANY(_user_ids));
        /* Ajoute les acces utilisateur etranger*/
        INSERT INTO clariweb_documentperuser
            (document_id, user_id, updated, "foreign", seen_since_modified)
        (SELECT
            _document_id,
            clariweb_clariwebuser.id,
            now(),
            _document.user_id != clariweb_clariwebuser.id,
            false
        FROM clariweb_clariwebuser WHERE id = ANY(_user_ids))
        ON CONFLICT DO NOTHING;
    END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION user_group_updated() RETURNS TRIGGER AS $$
    DECLARE
        _r record;
    BEGIN
        /* UPDATE all living documents on that user */
        FOR _r IN SELECT id FROM clariweb_document WHERE user_id = NEW.id
        LOOP
            PERFORM update_document_per_user(_r.id);
        END LOOP;
        IF TG_OP = 'UPDATE' AND (NEW.group IS DISTINCT FROM OLD.group) OR (TG_OP = 'INSERT') THEN
            /* Met a jour tous les documents qui concernent le nouveau groupe et l'ancien groupe de l'utilisateur */
            IF NEW.group IS NOT NULL AND NEW.group != '' THEN
                FOR _r IN SELECT clariweb_document.id
                    FROM clariweb_document
                    JOIN clariweb_clariwebuser ON (clariweb_clariwebuser.id = clariweb_document.user_id)
                    WHERE "group" = NEW.group
                LOOP
                    PERFORM update_document_per_user(_r.id);
                END LOOP;
            END IF;
            IF TG_OP = 'UPDATE' AND OLD.group IS NOT NULL AND OLD.group != '' THEN
                FOR _r IN SELECT clariweb_document.id
                    FROM clariweb_document
                    JOIN clariweb_clariwebuser ON (clariweb_clariwebuser.id = clariweb_document.user_id)
                    WHERE "group" = OLD.group
                LOOP
                    PERFORM update_document_per_user(_r.id);
                END LOOP;
            END IF;
        END IF;
        /* Met a jour tous les documents qui concernent l'ancienne et la nouvelle clinique de l'utilisateur */
        IF (TG_OP = 'UPDATE' AND NEW.group_clinic IS DISTINCT FROM OLD.group_clinic) OR (TG_OP = 'INSERT') THEN
            IF NEW.group_clinic IS NOT NULL AND NEW.group_clinic != '' THEN
                FOR _r IN SELECT clariweb_document.id
                    FROM clariweb_document
                    JOIN clariweb_clariwebuser ON (clariweb_clariwebuser.id = clariweb_document.user_id)
                    WHERE "group_clinic" = NEW.group_clinic
                LOOP
                    PERFORM update_document_per_user(_r.id);
                END LOOP;
            END IF;
            IF TG_OP = 'UPDATE' AND OLD.group_clinic IS NOT NULL AND OLD.group_clinic != '' THEN
                FOR _r IN SELECT clariweb_document.id
                    FROM clariweb_document
                    JOIN clariweb_clariwebuser ON (clariweb_clariwebuser.id = clariweb_document.user_id)
                    WHERE "group_clinic" = OLD.group_clinic
                LOOP
                    PERFORM update_document_per_user(_r.id);
                END LOOP;
            END IF;
        END IF;

        RETURN NULL;
    END;
$$ LANGUAGE plpgsql;


CREATE FUNCTION document_updated() RETURNS TRIGGER AS $$
    BEGIN
        PERFORM update_document_per_user(NEW.id);
        RETURN NULL;
    END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER trigger_clariwebuser_group_modified AFTER UPDATE
    ON clariweb_clariwebuser FOR EACH ROW
    WHEN (OLD.group IS DISTINCT FROM NEW.group OR OLD.group_clinic IS DISTINCT FROM NEW.group_clinic)
    EXECUTE PROCEDURE user_group_updated();

CREATE TRIGGER trigger_clariwebuser_group_added AFTER INSERT
    ON clariweb_clariwebuser FOR EACH ROW
    EXECUTE PROCEDURE user_group_updated();

CREATE TRIGGER trigger_clariwebuser_added AFTER INSERT
    ON clariweb_clariwebuser FOR EACH ROW
    EXECUTE PROCEDURE user_group_updated();

CREATE TRIGGER trigger_document_added AFTER INSERT
    ON clariweb_document FOR EACH ROW
    EXECUTE PROCEDURE document_updated();

CREATE TRIGGER trigger_document_modified AFTER UPDATE
    ON clariweb_document FOR EACH ROW
    WHEN (OLD.user_id IS DISTINCT FROM NEW.user_id)
    EXECUTE PROCEDURE document_updated();

CREATE FUNCTION update_all_documents() RETURNS VOID AS $$
    DECLARE
        _r record;
    BEGIN
        FOR _r IN SELECT id FROM clariweb_document ORDER BY id DESC LOOP
            PERFORM update_document_per_user(_r.id);
        END LOOP;
    END;
$$ LANGUAGE plpgsql;
SELECT * FROM update_all_documents();
DROP FUNCTION update_all_documents();
        """)
    ]
