"""
:copyright: (c) 2010-2014 by the Sentry Team
:copyright: (c) 2018 CLARISYS Informatique
:license: BSD, see LICENSE for more details.

"""

from __future__ import absolute_import

import os
import posixpath

from django.conf import settings
from django.http import HttpResponseNotFound, Http404
from django.contrib.staticfiles import finders
from django.utils.six.moves.urllib.parse import unquote
from django.views.decorators.csrf import get_token
from django.views import static


FOREVER_CACHE = 'max-age=315360000'
NEVER_CACHE = 'max-age=0, no-cache, no-store, must-revalidate'

def static_media(request, **kwargs):
    path = kwargs.get('path', '')
    response = static.serve(request, path, document_root=settings.WEBUI_ROOT)
    # Make sure we Vary: Accept-Encoding for gzipped responses
    response['Vary'] = 'Accept-Encoding'
    # We need CORS for font files
    if path.endswith(('.js', '.ttf', '.ttc', '.otf', '.eot', '.woff', '.woff2')):
        response['Access-Control-Allow-Origin'] = '*'
    if path.endswith(".html"):
        # Force getting the CSRF token cookie
        get_token(request)
        response['Cache-Control'] = NEVER_CACHE
    else:
        if settings.DEBUG:
            response['Cache-Control'] = NEVER_CACHE
        else:
            response['Cache-Control'] = FOREVER_CACHE
    return response
