# -*- coding: utf-8 -*-

import io
import gzip
from django.core.files.storage import FileSystemStorage
from django.core.files import File


class OverwriteFileSystemStorage(FileSystemStorage):
    def _save(self, name, content):
        self.delete(name)
        return super()._save(name, content)

    def get_available_name(self, name, max_length=None):
        return name


class GZFileSystemStorage(OverwriteFileSystemStorage):
    def _open(self, name, mode='rb'):
        return File(gzip.GzipFile(self.path(name), mode=mode))

    def _save(self, name, content):
        f = File(io.BytesIO(gzip.compress(content.read())), name)
        return super()._save(name, f)
