import angular from 'angular';
import bowser from 'bowser';

import * as Sentry from '@sentry/browser';

import '@babel/polyfill';
import '@uirouter/angularjs';
import 'angular-ui-bootstrap';
import 'angular-i18n/angular-locale_fr-fr';
import 'angular-cookies';
import 'angular-ui-bootstrap';
import 'angular-local-storage';
import 'angular-load';
import 'ng-tags-input';
// import { Visualizer } from '@uirouter/visualizer';
import { StickyStatesPlugin } from '@uirouter/sticky-states';
import './external/http-auth-interceptor';
import './services.js';
import './filters.js';
import './directives.js';
import './controllers.js';
import './controllers.documents.js';
import './controllers.clinic.js';
import './controllers.admin.js';
import './components.js';

require.ensure([], function() { require('../../less/main.less'); });

Sentry.init({
    dsn: 'https://3e47315a07be4dae9a8381d966068e8e@sentry.clarisys.fr/2',
    release: "clariweb@" + VERSION
});

angular.module('clariweb',
    [
        /* AngularJS dependencies */
        'ngCookies',
        'ngLocale',
        'ui.router',
        /* Clariweb dependencies */
        'clariweb.services',
        'clariweb.filters',
        'clariweb.directives',
        'clariweb.controllers',
        'clariweb.controllers.documents',
        'clariweb.controllers.clinic',
        'clariweb.controllers.admin',
        'clariweb.components',
        //'clariweb.controllers.test',
        /* External dependencies */
        'http-auth-interceptor',
        /* AngularJS ui */
        'ui.bootstrap',
        /* Angular local storage */
        'LocalStorageModule',
        /* Angular-load (async load) */
        'angularLoad',
        'ngTagsInput',
    ]
)

.config(
    function($stateProvider, $urlRouterProvider) {
        $urlRouterProvider.otherwise("/login");
        $stateProvider
            .state('login', {
                url: "/login?n&query_params&{i:int}&u",
                template: require('../partials/login.tpl.html'),
                controller: 'LoginCtrl',
                params: {
                    n: { value: null, squash: true },
                    query_params: { value: null, squash: true },
                    i: { value: null, squash: true },
                    u: { value: null, squash: true }
                }
            })
            .state('passwordreset', {
                url: "/passwordreset",
                template: require('../partials/passwordreset.tpl.html'),
                controller: 'PasswordResetCtrl'
            })
            // TODO: Replace with proper $stateParams
            .state('emailconfirm', {
                url: '/emailconfirm?key',
                template: require('../partials/emailconfirm.tpl.html'),
                controller: 'EmailConfirm'
            })
            .state('document', {
                abstract: true,
                url: '/document',
                template: require('../partials/document.tpl.html')
            })
            .state('document.list', {
                url: '/list/?:filter&:{showarchive:bool}',
                params: {
                    filter: { dynamic: true, value: null },
                    showarchive: { dynamic: true, value: true }
                },
                views: {
                    list: {
                        controller: 'DocumentListCtrl',
                        template: require('../partials/list.tpl.html')
                    },
                },
                sticky: true
            })
            .state('document.detail', {
                url: '/detail/{document_id:int}',
                views: {
                    detail: {
                        template: require('../partials/detail.tpl.html'),
                        controller: 'DocumentDetailCtrl'
                    }
                }
            })
            .state('clinic', {
                abstract: true,
                url: '/clinic',
                template: require('../partials/clinic.tpl.html'),
                controller: 'ClinicCtrl'
            })
            .state('clinic.list', {
                url: '/list/?:date&:lastname&:firstname&:clinic_hospi&:clinic_ipp&:shareservices&:clinic_service',
                params: {
                    date: { dynamic: true, value: "1" },
                    lastname: { dynamic: true },
                    firstname: { dynamic: true },
                    clinic_hospi: { dynamic: true },
                    clinic_ipp: { dynamic: true },
                    clinic_service: { dynamic: true },
                    shareservices: { dynamic: true }
                },
                views: {
                    list: {
                        controller: 'ClinicDocumentListCtrl',
                        template: require('../partials/clinic.list.tpl.html')
                    }
                },
                sticky: true
            })
            .state('clinic.detail', {
                url: '/detail/{document_id:int}',
                views: {
                    detail: {
                        controller: 'ClinicDocumentDetailCtrl',
                        template: require('../partials/clinic.detail.tpl.html')
                    }
                }
            })
            .state('debpt', {
                abstract: true,
                url: '/debpt',
                template: '<div ui-view></div>'
            })
            .state('debpt.info', {
                url: '/info',
                controller: 'DebptInfoCtrl',
                template: require('../partials/debpt.info.tpl.html')
            })
            .state('payment', {
                url: '/payment/{status:.*}',
                controller: 'PaymentReturnController',
                template: require('../partials/payment.return.tpl.html'),
                params: {
                    query_params: null
                }
            })
            .state('paymentlist', {
                url: '/paymentlist',
                controller: 'PayementRequestListController',
                template: require('../partials/payment.list.tpl.html'),
            })
            .state('settings', {
                url: '/settings',
                template: require('../partials/settings.tpl.html'),
                controller: 'SettingsCtrl'
            })
            .state('settings.sms', {
                url: '/sms',
                template: require('../partials/settings.sms.tpl.html'),
                controller: 'AdminSMSCtrl'
            })
            .state('settings.notifications', {
                url: '/notifications',
                template: require('../partials/settings.notifications.tpl.html'),
                controller: 'AdminNotificationsCtrl'
            })
            .state('settings.payments', {
                url: '/payments',
                template: require('../partials/settings.payments.tpl.html'),
                controller: 'AdminPaymentsCtrl'
            })
            .state('settings.accesslog', {
                url: '/accesslog',
                template: require('../partials/settings.accesslog.tpl.html'),
                controller: 'AdminAccessLogCtrl'
            })
            .state('settings.loginas', {
                url: '/loginas',
                template: require('../partials/settings.loginas.tpl.html'),
                controller: 'AdminLoginAsCtrl'
            })
            .state('settings.settings', {
                url: '/settings',
                template: require('../partials/settings.settings.tpl.html'),
                controller: 'AdminSettingsCtrl'
            })
            .state('settings.stats', {
                url: '/stats',
                template: require('../partials/settings.stats.tpl.html'),
                controller: 'AdminStatsCtrl'
            })
            .state('logout', {
                url: '/logout',
                template: require('../partials/logout.tpl.html'),
                controller: 'LogoutCtrl'
            })
            .state('contact', {
                url: '/contact',
                template: require('../partials/contact.tpl.html'),
                controller: 'ContactCtrl'
            })
            .state('error', {
                url: '/error',
                template: require('../partials/error.tpl.html'),
                controller: 'ErrorCtrl',
                data: {}
            })
            .state('error.serverdown', {
                url: '/serverdown',
            })
            ;
    }
)

.config(
    function($httpProvider, $uiRouterProvider, $qProvider) {
        $qProvider.errorOnUnhandledRejections(false);
        $uiRouterProvider.plugin(StickyStatesPlugin);
        // $uiRouterProvider.plugin(Visualizer);
        $httpProvider.interceptors.push(
            ["$q", "$rootScope", "ClariwebLoading",
            function($q, $rootScope, ClariwebLoading) {
                return {
                    request: function(config) {
                        ClariwebLoading.load();
                        return config || $q.when(config);
                    },
                    response: function(response) {
                        ClariwebLoading.done();
                        return response || $q.when(response);
                    },
                    responseError: function(rejection) {
                        ClariwebLoading.done();
                        if ((rejection.status === 500) || (rejection.status === 502)) {
                            $rootScope.$emit('event:error-serverdown');
                        }
                        return $q.reject(rejection);
                    }
                };
            }]
        );
    }
)

.run(
    function run($http, $rootScope, $state, $stateParams, $interval, $location,
                 $httpParamSerializer, $window, $transitions,
                 ClariwebAuth) {

        /* Check old browser support */
        let browser = bowser.getParser(window.navigator.userAgent);
        if (browser.satisfies({
            "internet explorer": "<10",
            chrome: "<35",
            firefox: "<13",
            opera: "<13",
            safari: "<6"
        })) {
            /* Browser too old does not support new Blob() */
            // NOTE: DO NOT use require or anything else, this is hardcoded
            $window.location.href = "assets/oldie.html";
        }

        var except_login_state = [
            'login', 'logout', 'emailconfirm', 'error', 'error.serverdown',
            'contact', 'passwordreset' ];
        $http.defaults.xsrfCookieName = "csrftoken";
        $http.defaults.xsrfHeaderName = "X-CSRFToken";
        $rootScope.auth = ClariwebAuth;
        $rootScope.$state = $state;
        $rootScope.$on('event:error-serverdown', function() {
            $state.go("error.serverdown");
        });
        /* TODO: Move this to a service */
        /* Automatic logout when no activity */
        var resetTimeoutChecker = function() {
            $rootScope.nexttimeout = new Date().getTime() + ClariwebAuth.inactivityTimeout;
        };
        var enableTimeoutChecker = function() {
            if (! angular.isDefined($rootScope.timeoutchecker)) {
                resetTimeoutChecker();
                $rootScope.timeoutchecker = $interval(function() {
                    $rootScope.timeoutin = $rootScope.nexttimeout - new Date().getTime();
                    if ($rootScope.timeoutin <= 0) {
                        ClariwebAuth.logout();
                        $state.go("login");
                    }
                }, 1000);
            }
        };
        var cancelTimeoutChecker = function() {
            if (angular.isDefined($rootScope.timeoutchecker)) {
                $interval.cancel($rootScope.timeoutchecker);
                $rootScope.timeoutchecker = undefined;
            }
        };
        var checkEnableTimeoutChecker = function() {
            if (ClariwebAuth.inactivityTimeout > (600 * 1000)) {
                cancelTimeoutChecker();
            } else {
                if (ClariwebAuth.loggedin) {
                    enableTimeoutChecker();
                } else {
                    cancelTimeoutChecker();
                }
            }
        };
        var inactivityTimeoutChanged = function() {
            checkEnableTimeoutChecker();
            resetTimeoutChecker();
        };
        $rootScope.$watch('auth.inactivityTimeout', inactivityTimeoutChanged);
        $transitions.onBefore({}, function(trans) {
            var auth = trans.injector().get('ClariwebAuth');
            var tostate = trans.to().name;
            if (except_login_state.indexOf(tostate) === -1 && !auth.loggedin) {
                return auth.try_autologin().then(() => {
                    if (! auth.loggedin) {
                        return trans.router.stateService.target('login');
                    }
                });
            }
        });
        $transitions.onSuccess({}, () => { resetTimeoutChecker(); });
        $rootScope.$on('clariweb_activity', resetTimeoutChecker);
        $rootScope.$watch('auth.loggedin', checkEnableTimeoutChecker);
    }
)
;