angular.module('clariweb.controllers.admin', [])
.controller(
    'AdminLoginAsCtrl',
    function(ClariwebAuth, Constants, $scope, $http, $state) {
        $scope.inLoad = false;
        $scope.user = undefined;
        $scope.users = [];
        $scope.totalUsers = 0;
        $scope.currentPage = 1;
        $scope.pagesize = 50;
        $scope.$watch('filter', function(newfilter, oldfilter) {
            $scope.currentPage = 1;
            if (newfilter !== oldfilter && newfilter !== '') {
                $scope.fetch();
            }
        });
        $scope.$watch('currentPage', function(newpage, oldpage) {
            if (oldpage !== newpage && angular.isDefined($scope.filter) && $scope.filter) {
                $scope.fetch();
            }
        });
        $scope.fetch = function fetch() {
            if ($scope.inLoad) {
                return;
            }
            $scope.inLoad = true;
            $http({
                method: 'GET',
                url: Constants.API_ADMIN_LOGINAS_SELECTOR,
                params: {
                    search: $scope.filter,
                    page: $scope.currentPage
                }
            })
            .then(
                function(response) {
                    $scope.users = response.data.results;
                    $scope.totalUsers = response.data.results.length + (
                        (response.data.page-1) * response.data.pagesize) + 1;
                    $scope.pagesize = response.data.pagesize;
                    $scope.inLoad = false;
                },
                function() {
                    $scope.inLoad = false;
                }
            );
        };
        $scope.setUser = function(user) {
            $scope.user = user;
            $scope.errors = undefined;
            $scope.newpassword = '';
            if (! user) {
                $scope.selected = undefined;
            }
        };
        $scope.loginAsUser = function(username) {
            ClariwebAuth.loginas(username).then(function() {
                $state.go("document.list");
            });
        };
        $scope.changepassword = function(user) {
            $scope.errors = undefined;
            if (user) {
                var password = Math.random().toString(36).slice(-12);
                $http.post(Constants.API_ADMIN_USERMODIFY,
                    {
                        username: user.user.username,
                        password: password
                    }
                ).then(
                    function(response) {
                        $scope.user = response.data;
                        $scope.newpassword = password;
                    },
                    function(response) {
                        $scope.errors = response.data;
                    }
                );
            }
        };
        $scope.setactive = function(user, active) {
            $scope.errors = undefined;
            if (user) {
                var q = $http.post(Constants.API_ADMIN_USERMODIFY,
                    {
                        username: user.user.username,
                        is_active: active
                    }
                );
                q.then(
                    function(response) {
                        $scope.user = response.data;
                    },
                    function(response) {
                        $scope.errors = response.data.errors;
                    }
                );
                return q;
            }
        };
    }
)

.controller(
    'AdminSMSCtrl',
    function(Constants, $scope, $http, $uibModal, ClariwebLoading) {
        $scope.smscount_total = 0;
        $scope.sms_price = Constants.SMS_PRICE;
        $scope.smsquota = -1;
        $scope.errors = "";
        $scope.$watch('smsquota', function(smsquota) {
            $scope.smscountwarning = (smsquota < 100);
            $scope.smscountgood = (smsquota >= 1000);
        });
        $http.get(Constants.API_ADMIN_SMSCOUNT).then(function(response) {
            $scope.smscount_total = response.data.smscount_total;
        });
        $scope.ClariwebLoading = ClariwebLoading;
        $http.get(Constants.API_ADMIN_SMSCOUNT_QUOTA).then(function(response) {
            $scope.smsquota = response.data.sms_quota;
        });
        var modalInstanceCtrl = ['$scope', '$uibModalInstance', 'smscount',
            function($scope, $uibModalInstance, smscount) {
            $scope.smscount = smscount;
            $scope.command_sent = false;
            $scope.error = '';
            $scope.buy = function() {
                $http.post(Constants.API_ADMIN_BUYSMS,
                           {'quantity': $scope.smscount})
                .then(
                    function() {
                        $scope.command_sent = true;
                    },
                    function(response) {
                        $scope.error = response.data;
                    }
                );
            };
            $scope.close = function() {
                $uibModalInstance.close();
            };
            $scope.cancel = function() {
                $uibModalInstance.dismiss('cancel');
            };
        }];

        $scope.updatesmsquota = function() {
            $scope.errors = "";
            $http.get(Constants.API_ADMIN_SMSCOUNT_UPDATEQUOTA)
            .then(
                function(response) {
                    $scope.smsquota = response.data.sms_quota;
                },
                function(response) {
                    $scope.errors = "Erreur lors de la récupération du quota: " + response.data.errors;
                });
        };
        $scope.buysms = function(quantity) {
            $uibModal.open({
                template: require('../partials/settings.smsbuy.tpl.html'),
                controller: modalInstanceCtrl,
                resolve: {
                    smscount: function() {
                        return quantity;
                    }
                }
            });
        };
    }
)

.controller(
    'AdminNotificationsCtrl',
    function(Constants, $scope, $http) {
        $scope.inLoad = false;
        $scope.notifications = [];
        $scope.order = 'date';
        $scope.reverse = true;
        $scope.currentPage = 1;
        $scope.pagesize = 50;
        $scope.totalNotifications = 0;
        $scope.$watch('filter', function() {
            $scope.currentPage = 1;
            //$scope.checkerror = (filter && filter.indexOf("ERROR") != -1);
            $scope.fetch();
        });
        $scope.$watch('order', function() {
            $scope.fetch();
        });
        $scope.$watch('reverse', function() {
            $scope.fetch();
        });
        $scope.$watch('currentPage', function() {
            $scope.fetch();
        });
        $scope.$watch('checkerror', function(checkerror) {
            if (checkerror) {
                var filter = "ERROR" + ($scope.filter ? (' ' + $scope.filter) : '');
                $scope.filter = filter;
            } else {
                $scope.filter = '';
            }
        });
        $scope.fetch = function fetch() {
            if ($scope.inLoad) {
                return;
            }
            $scope.inLoad = true;
            $http({
                method: 'GET',
                url: Constants.API_ADMIN_NOTIFICATION,
                params: {
                    search: $scope.filter,
                    ordering: ($scope.reverse ? '-' : '') + $scope.order,
                    page: $scope.currentPage
                }
            })
            .then(
                function(response) {
                    $scope.notifications = response.data.results;
                    $scope.totalNotifications = response.data.results.length + (
                        (response.data.page-1) * response.data.pagesize) + 1;
                    $scope.pagesize = response.data.pagesize;
                    $scope.inLoad = false;
                },
                function() {
                    $scope.inLoad = false;
                }
            );
        };
        $scope.fetch();
    }
)

.controller(
    'AdminPaymentsCtrl',
    function(Constants, $scope, $http) {
        $scope.inLoad = false;
        $scope.payments = [];
        $scope.order = 'pzrequest.date_insert';
        $scope.reverse = true;
        $scope.currentPage = 1;
        $scope.pagesize = 50;
        $scope.totalPayments = 0;
        $scope.$watch('filter', function() {
            $scope.currentPage = 1;
            $scope.fetch();
        });
        $scope.$watch('order', function() {
            $scope.fetch();
        });
        $scope.$watch('reverse', function() {
            $scope.fetch();
        });
        $scope.$watch('currentPage', function() {
            $scope.fetch();
        });
        $scope.fetch = function fetch() {
            if ($scope.inLoad) {
                return;
            }
            $scope.inLoad = true;
            $http({
                method: 'GET',
                url: Constants.API_ADMIN_PAYMENTS,
                params: {
                    search: $scope.filter,
                    ordering: ($scope.reverse ? '-' : '') + $scope.order,
                    page: $scope.currentPage
                }
            })
            .then(
                function(response) {
                    $scope.payments = response.data.results;
                    $scope.pagesize = response.data.pagesize;
                    $scope.totalPayments = response.data.results.length + (
                        (response.data.page-1) * response.data.pagesize) + 1;
                    $scope.inLoad = false;
                },
                function() {
                    $scope.inLoad = false;
                }
            );
        };
        $scope.fetch();
    }
)

.controller(
    'AdminAccessLogCtrl',
    function(Constants, $scope, $http) {
        $scope.inLoad = false;
        $scope.accesslogs = [];
        $scope.currentPage = 1;
        $scope.totalLogs = 0;
        $scope.pagesize = 50;
        $scope.$watch('filter', function() {
            $scope.currentPage = 1;
            $scope.fetch();
        });
        $scope.$watch('currentPage', function() {
            $scope.fetch();
        });
        $scope.fetch = function fetch() {
            if ($scope.inLoad) {
                return;
            }
            $scope.inLoad = true;
            $http({
                method: 'GET',
                url: Constants.API_ADMIN_ACCESSLOG,
                params: {
                    search: $scope.filter,
                    page: $scope.currentPage
                }
            })
            .then(
                function(response) {
                    $scope.accesslogs = response.data.results;
                    $scope.totalLogs = response.data.results.length + (
                        (response.data.page-1) * response.data.pagesize) + 1;
                    $scope.pagesize = response.data.pagesize;
                    $scope.inLoad = false;
                },
                function() {
                    $scope.inLoad = false;
                }
            );
        };
        $scope.fetch();
    }
)

.controller(
    'AdminSettingsCtrl',
    function(Constants, $scope, $http, $uibModal, ClariwebThemer) {
        $scope.items = [];
        $scope.themer = ClariwebThemer;
        $scope.reload = function() {
            $scope.diff = [];
            $http.get(Constants.API_ADMIN_SETTINGS).then(function(response) {
                /* data needs to be something like:
                 * [{key: "TOTO", help_text: "blabla blabla", value: "XX"}]
                 */
                var items = response.data;
                var i;
                for(i = 0; i < items.length; i++) {
                    items[i].ro_value = angular.copy(items[i].value);
                    if (items[i].type === 'iplist') {
                        items[i].tags = angular.copy(items[i].value);
                    }
                }
                $scope.items = items;
            });
        };

        $scope.isTextInput = function(item) {
            return (['number', 'text', 'password'].indexOf(item.type) !== -1);
        };

        $scope.isAnyDiff = function(item) {
            return angular.equals(item.ro_value, item.value);
        };

        $scope.$watch("items", function(items) {
            $scope.diff = [];
            var i;
            for (i = 0; i < items.length; i++) {
                var item = items[i];
                var value = item.value;
                if (item.type === 'iplist' && angular.isDefined(item.tags)) {
                    value = [];
                    for (var j = 0; j < item.tags.length; j++) {
                        value.push(item.tags[j].text);
                    }
                    item.value = value;
                }
                if (!angular.equals(value, item.ro_value)) {
                    $scope.diff.push(item);
                }
            }
        }, true);
        var modalInstanceCtrl = ['$scope', '$uibModalInstance', 'diff',
            function($scope, $uibModalInstance, diff) {
            $scope.diff = diff;
            $scope.saved = false;
            $scope.error = '';
            $scope.realsave = function() {
                var tosave = [];
                var i;
                for(i=0; i < $scope.diff.length; i++) {
                    tosave.push({
                        key: $scope.diff[i].key,
                        value: $scope.diff[i].value
                    });
                }
                $http.post(Constants.API_ADMIN_SETTINGS, {
                    items: tosave})
                .then(
                    function() {
                        $scope.saved = true;
                    },
                    function(response) {
                        $scope.error = response.data;
                        $scope.saved = false;
                    }
                );
            };
            $scope.close = function() {
                $uibModalInstance.close();
            };
            $scope.cancel = function() {
                $uibModalInstance.dismiss('cancel');
            };
        }];
        $scope.save = function() {
            $uibModal.open({
                template: require('../partials/settings.settings.savediff.tpl.html'),
                controller: modalInstanceCtrl,
                resolve: {
                    diff: function() {
                        return $scope.diff;
                    }
                }
            }).result.then(function() {
                $scope.reload();
            });
        };
        $scope.reload();
    }
)

.controller(
    'AdminStatsCtrl',
    function(Constants, ClariwebAuth, $scope, $http, $state) {
        $scope.inLoad = false;
        $scope.stats = [];
        $scope.currentPage = 1;
        $scope.totalStats = 0;
        $scope.chartObj = undefined;
        $scope.$watch('filter', function() {
            $scope.currentPage = 1;
            $scope.fetch();
        });
        $scope.$watch('currentPage', function() {
            $scope.fetch();
        });
        $scope.onselectpage = function(page) {
            $scope.currentPage = page;
        };
        $scope.loginAsUser = function(username) {
            ClariwebAuth.loginas(username).then(function() {
                $state.go("document.list");
            });
        };
        $scope.fetch = function fetch() {
            if ($scope.inLoad) {
                return;
            }
            $scope.inLoad = true;
            $http({
                method: 'GET',
                url: Constants.API_ADMIN_STATS_MONTH_USER,
                params: {
                    page: $scope.currentPage
                }
            })
            .then(
                function(response) {
                    $scope.stats = response.data.results;
                    $scope.totalStats = response.data.count;
                    $scope.inLoad = false;
                },
                function() {
                    $scope.inLoad = false;
                }
            );
        };
        $scope.fetchgraph = function fetchgraph() {
            $http({
                method: 'GET',
                url: Constants.API_ADMIN_STATS_MONTH_GRAPH,
            }).then(function(response) {
                var series_map = {"PAT": 0, "MED": 1, "COR": 2, "PREL": 3};
                var series = [
                    {
                        name: "Patient",
                        data: []
                    },
                    {
                        name: "Médecin",
                        data: []
                    },
                    {
                        name: "Correspondant",
                        data: []
                    },
                    {
                        name: "Préleveur",
                        data: []
                    }
                ];
                for (var i=0; i < response.data.length; i++) {
                    if (response.data[i].userkind) {
                        var serie = series[series_map[response.data[i].userkind]];
                        serie.data.push([
                            Date.parse(response.data[i].timestep),
                            response.data[i].documents
                        ]);
                    }
                }
                $scope.graphPerMonth = {
                    chart: {type: 'column'},
                    title: {text: 'Quantité de documents ajoutés par mois'},
                    xAxis: {type: 'datetime'},
                    yAxis: {title: { text: 'documents'} },
                    series: series,
                    plotOptions: {column: {stacking: 'normal'}},
                    legend: {enabled: true}
                };
            });
        };
        $scope.fetch();
        $scope.fetchgraph();
    }
)

;
