const REFRESH_SECONDS = 300;

angular.module('clariweb.controllers.clinic', [])

.controller('ClinicCtrl',
    function(ClariwebCustomSettings, $state) {
        if (! ClariwebCustomSettings.get("CLINIC_ENABLED")) {
            $state.go("document.list");
        }
    }
)

.controller(
    'ClinicDocumentListCtrl',
    function(ClariwebAuth, Constants, $scope, $http, $state, $interval,
             $location, $anchorScroll) {
        $scope.filters = {};
        $scope.date = '1';
        $scope.can_autorefresh = true;
        let refreshtimer = null;

        function getfilters() {
            return {
                date: parseInt($scope.date),
                lastname: $scope.lastname,
                firstname: $scope.firstname,
                clinic_hospi: $scope.clinic_hospi,
                clinic_ipp: $scope.clinic_ipp,
                clinic_service: $scope.clinic_service,
                shareservices: $scope.shareservices
            };
        }
        $scope.onCancel = function() {
            $scope.lastname = '';
            $scope.firstname = '';
            $scope.clinic_hospi = '';
            $scope.clinic_ipp = '';
            $scope.clinic_service = '';
            $scope.enable_autorefresh = true;
            $scope.can_autorefresh = true;
            $scope.shareservices = true;
            $scope.document_id = null;
            $scope.onSearch();
        };
        $scope.onSearch = function() {
            let active_filters = getfilters();
            $scope.updateState().then(function(){
                if (! angular.equals(active_filters, $scope.active_filters)) {
                    $scope.fetch(active_filters, $scope.currentPage);
                }
            });
        };
        $scope.updateState = function() {
            return $state.go("clinic.list", getfilters());
        };
        $scope.canSearchOlder = function() {
            return $scope.date !== '0';
        };
        $scope.searchOlder = function() {
            if (! $scope.canSearchOlder()) {
                return;
            }
            if ($scope.date === '1') {
                $scope.date = '3';
            } else if ($scope.date === '3') {
                $scope.date = '7';
            } else if ($scope.date === '7') {
                $scope.date = '30';
            } else if ($scope.date === '30') {
                $scope.date = '0';
            } else {
                return;
            }
            $scope.onSearch();
        };

        $scope.inLoad = false;
        $scope.currentPage = 1;
        $scope.$watch('currentPage', function(page, oldpage) {
            if (page !== oldpage) {
                $scope.fetch($scope.filters, page);
            }
        });
        $scope.onselectpage = function(page) {
            $scope.currentPage = page;
        };
        $scope.selectitem = function(doc) {
            doc.seen_since_modified = true;
            doc.last_seen = new Date();
            $state.go("clinic.detail", {document_id: doc.document.id});
        };
        $scope.fetch = function(filters, page) {
            if ($scope.inLoad) {
                return;
            }
            $scope.inLoad = true;
            let params = {page: page};
            for (let key in filters) {
                if (filters[key] && key !== "shareservices") {
                    params[key] = filters[key];
                }
            }
            let apiurl = Constants.API_CLINIC_DOCUMENT_LIST;
            if (! $scope.shareservices) {
                apiurl = Constants.API_CLINIC_DOCUMENT_LIST_NOSHARE;
            }
            $http({
                method: 'GET', url: apiurl, params: params
            }).then(
                function(response) {
                    $scope.refresh_tick_count = REFRESH_SECONDS;
                    let documents = [];
                    for(let i in response.data.results) {
                        let document = response.data.results[i];
                        if (! document.document.analyses) {
                            document.document.analyses = [];
                        }
                        documents.push(document);
                    }
                    $scope.documents = documents;
                    $scope.inLoad = false;
                    $location.hash("body");
                    $anchorScroll();
                },
                function(response) {
                    $scope.refresh_tick_count = REFRESH_SECONDS;
                    if (response.status === 403 || response.status === 401) {
                        $state.go("logout");
                    } else {
                        $scope.documents = [];
                    }
                    $scope.inLoad = false;
                }
            );
        };
        $scope.refresh_tick_count = REFRESH_SECONDS;
        /* LC: beware of the order, we want the watch to catch initial $state params */
        $scope.$watchGroup(["lastname", "firstname", "clinic_hospi", "clinic_ipp"], () => {
            $scope.can_autorefresh = (!$scope.lastname && !$scope.firstname && !$scope.clinic_hospi && !$scope.clinic_ipp);
            $scope.enable_autorefresh = $scope.can_autorefresh;
        });
        $scope.enable_autorefresh = false;
        $scope.firstname = $state.params.firstname;
        $scope.lastname = $state.params.lastname;
        $scope.date = $state.params.date;
        $scope.clinic_hospi = $state.params.clinic_hospi;
        $scope.clinic_ipp = $state.params.clinic_ipp;
        $scope.clinic_service = $state.params.clinic_service;
        $scope.shareservices = $state.params.shareservices ? JSON.parse($state.params.shareservices) : true;
        $scope.filters = getfilters();
        $scope.fetch($scope.filters, 1);
        $scope.$watch("enable_autorefresh", () => {
            $scope.refresh_tick_count = REFRESH_SECONDS;
        });
        $scope.$watch("shareservices", (shareservices) => {
            if (!shareservices) {
                $scope.clinic_service = '';
            }
        });
        $scope.$on("$destroy", () => {
            if (refreshtimer) {
                $interval.cancel(refreshtimer);
                refreshtimer = null;
            }
        });
        refreshtimer = $interval(() => {
            if (! $scope.enable_autorefresh) {
                return;
            }
            $scope.refresh_tick_count -= 1;
            if ($scope.refresh_tick_count <= 0) {
                $scope.refresh_tick_count = REFRESH_SECONDS;
                $scope.fetch($scope.filters, 1);
            }
        }, 1000);
    }
)
.controller(
    'ClinicDocumentDetailCtrl',
    function($http, $scope, $state, $rootScope, $transition$, Constants) {
        $scope.analyses = [];
        $scope.error = false;
        $scope.inload = false;

        function sortByDateAndDemande(list) {
            if (! angular.isDefined(list) || list === []) {
                return [];
            }
            list.sort(function(a, b) {
                let cmp = b.date - a.date;
                return cmp ? cmp : b.numero.localeCompare(a.numero);
            });
        }

        function buildAnterioritesHeader(resultats) {
            var anteriorites_header = [];
            for(let resultat of resultats) {
                if (! angular.isDefined(resultat.anteriorites)) {
                    continue;
                }
                for(let anteriorite of resultat.anteriorites) {
                    anteriorite.date = new Date(anteriorite.date);
                    let header = {
                        'date':  anteriorite.date,
                        'numero': anteriorite.numero
                    };
                    if (anteriorites_header.length > 0) {
                        var found = 0;
                        for (let element of anteriorites_header) {
                            if ((element.numero === anteriorite.numero) &&
                               (element.date - anteriorite.date) === 0) {
                                found = 1;
                                break;
                            }
                        }
                        if (found === 0) {
                            anteriorites_header.push(header);
                        }
                    } else {
                        anteriorites_header.push(header);
                    }
                }
            }
            sortByDateAndDemande(anteriorites_header);
            return anteriorites_header;
        }

        function buildAnteriorites(anteriorites, header) {
            var _anteriorites = [];
            var index = 0;
            sortByDateAndDemande(anteriorites);
            header.forEach(function(_header) {
                if ((index < anteriorites.length &&
                    _header.numero === anteriorites[index].numero &&
                    (_header.date - anteriorites[index].date) === 0)
                ) {
                    _anteriorites.push({
                        "valeur": anteriorites[index].valeur
                    });
                    index++;
                } else {
                    _anteriorites.push({
                        "valeur": ""
                    });
                }
            });
            return _anteriorites;
        }

        function postprocessAnalyses(analyses) {
            var processed_analyses = [];
            for(let analyse of analyses) {
                let anteriorites_header = buildAnterioritesHeader(analyse.resultats);
                let resultats = [];
                let anteriorites_index = [];
                for (let resultat of analyse.resultats) {
                    let anteriorites = buildAnteriorites(resultat.anteriorites, anteriorites_header);
                    let indication_normalite = {};
                    if (resultat.indication_normalite !== null) {
                        indication_normalite = {
                            "low": resultat.indication_normalite.low,
                            "high": resultat.indication_normalite.high,
                            "interpretation": resultat.indication_normalite.interpretation
                        };
                    }
                    resultats.push({
                        "is_compterendu_seulement": resultat.compterendu_seulement,
                        "is_resultat_masque": resultat.resultat_masque,
                        "is_resultat_valide": resultat.etat >= 3,
                        "etat": resultat.etat,
                        "indication_normalite": indication_normalite,
                        "testbio": {
                            "code": resultat.testbio.code,
                            "libelle": resultat.testbio.libelle
                        },
                        "unite": resultat.unite,
                        "type": resultat.type,
                        "valeur": resultat.valeur,
                        "anteriorites": anteriorites
                    });
                }
                for (let i in anteriorites_header) {
                    anteriorites_index.push(i);
                }
                processed_analyses.push({
                    "libelle_chapitre": analyse.chapitre.libelle,
                    "code_chapitre": analyse.chapitre.code,
                    "libelle_analyse": analyse.analyse.libelle,
                    "code_analyse": analyse.analyse.code,
                    "resultats": resultats,
                    "anteriorites_header": anteriorites_header,
                    "anteriorites_index": anteriorites_index
                });
            }
            return processed_analyses;
        }

        function postprocessDetails(document) {
            var details_patient = {
                "last_name": document.patient.last_name,
                "first_name": document.patient.first_name,
                "civilite": document.patient.civilite,
                "date_naissance": document.patient.date_naissance
            };

            var details_demande = {
                "numero": document.sil_demande_number,
                "date": document.sil_demande_date,
                "demandeur": document.correspondant,
                "medecin": document.medecin
            };

            return {
                "patient": details_patient,
                "demande": details_demande,
                "analyses": document.analyses,
            };
        }

        $scope.fetch = function(document_id) {
            $scope.document = null;
            $scope.download_url = '';
            $scope.analyses = [];
            $scope.details = {};
            $scope.error = false;
            if ($scope.inload) {
                return;
            }
            $scope.inload = true;
            $http.get(Constants.API_CLINIC_DOCUMENT_DETAIL.replace("{document_id}", document_id)
            ).then(function (response) {
                var document = response.data;
                $scope.download_url =
                    Constants.API_DOCUMENT_DOWNLOAD.replace("{document_id}",
                        document.id);
                try {
                    $scope.details = postprocessDetails(document);
                    $scope.analyses = postprocessAnalyses(document.structured_results);
                } catch (e) {
                    $scope.error = true;
                    console.log(e);
                }
                $scope.inload = false;
            }, function (e) {
                $scope.inload = false;
                $scope.error = true;
                console.log(e);
            });
        };
        $scope.show_resultats_masques = false;
        $scope.afficherResultatsCaches = function() {
            $scope.show_resultats_masques = true;
        };
        $scope.$watch("document_id", function(document_id) {
            $scope.fetch(document_id);
        });
        $scope.goback = function() {
            if ($transition$.$from().name) {
                $state.go($transition$.$from().name, $transition$.params('from'));
            } else {
                $state.go("clinic.list");
            }
        };
        $scope.document_id = $state.params.document_id;
    }
);
