angular.module('clariweb.controllers', [])
.controller(
    'MainCtrl',
    function(ClariwebAuth, ClariwebCustomSettings, ClariwebThemer,
             $http, $scope, $sce, $state) {
        $scope.apploading = true;
        $scope.isCollapsed = true;
        $scope.$on('event:auth-loginRequired', function() {
            ClariwebAuth.reset();
            $state.go("login");
        });
        /* load customization settings */
        ClariwebCustomSettings.load().then(function() {
            $scope.settings = ClariwebCustomSettings.settings;
            var customlogo = ClariwebCustomSettings.get("CUSTOM_LOGO");
            if (customlogo) {
                $scope.customlogo = $sce.trustAsResourceUrl(customlogo);
            }
            var theme_key = ClariwebCustomSettings.get("CUSTOM_THEME_KEY");
            $http.get("themes-manifest.json").then(function(resp) {
                ClariwebThemer.setStyles(resp.data);
                if (theme_key) {
                    ClariwebThemer.setSelected(theme_key);
                }
            });
            $scope.apploading = false;
        });
        $scope.transcodeStateName = function(name) {
            var transcodage = {
                'd.l': 'document.list'
            };
            if (transcodage[name] !== undefined) {
                return transcodage[name];
            }
            return name;
        };
        $scope.gohome = function() {
            if (ClariwebAuth.isclinic() && ClariwebCustomSettings.get("CLINIC_ENABLED")) {
                $state.go("clinic.list");
            } else {
                $state.go("document.list");
            }
        };
    }
)

.controller(
    'LoginCtrl',
    function(ClariwebAuth, ClariwebCustomSettings,
             $scope, $state, $stateParams, $httpParamSerializer) {
        $scope.login_error = false;
        $scope.login_invoice_error = false;
        $scope.auth = ClariwebAuth;
        $scope.asksetsessionexpiry = false;
        $scope.underaged = false;
        $scope.username = $stateParams.u;
        $scope.$watch('auth.user', function() {
            if (! ClariwebAuth.loggedin && ($scope.username || $scope.password)) {
                $scope.error = true;
            }
            if (ClariwebAuth.loggedin && !$scope.asksetsessionexpiry) {
                var next = $scope.transcodeStateName($stateParams.n);
                if (angular.isDefined(next) && next && next !== "login") {
                    if (next === "document.list") {
                        $state.go(next, {
                            'query_params': $stateParams.query_params,
                            'document_id': $stateParams.i
                        });
                    } else {
                        $state.go(next, $stateParams.query_params);
                    }
                } else {
                    if (ClariwebAuth.isclinic() && ClariwebCustomSettings.get("CLINIC_ENABLED")) {
                        $state.go("clinic.list");
                    } else {
                        $state.go("document.list");
                    }
                }
            }
        });
        $scope.reset = function() {
            $scope.username = '';
            $scope.password = '';
            $scope.birthdate = null;
            $scope.invoice_id = '';
            $scope.underaged = false;
            ClariwebAuth.reset();
            $scope.login_error = false;
            $scope.login_invoice_error = false;
        };
        $scope.submit = function() {
            $scope.login_error = false;
            $scope.login_invoice_error = false;
            ClariwebAuth.login($scope.username, $scope.password)
                .then(
                    function(response) {
                        $scope.asksetsessionexpiry = ClariwebAuth.is_session_supported() && response.data.setsessionexpiry;
                    },
                    function(response) {
                        if (response.status === 403) {
                            $scope.underaged = true;
                        } else {
                            $scope.login_error = true;
                        }
                    }
                );
        };
        $scope.submit_invoice = function() {
            $scope.login_error = false;
            $scope.login_invoice_error = false;
            ClariwebAuth.login_by_invoice($scope.invoice_id, $scope.birthdate)
                .then(
                    function() {
                        $scope.asksetsessionexpiry = false;
                    },
                    function() {
                        $scope.login_invoice_error = true;
                    }
                );
        };
        $scope.setsessionexpiry = function(val) {
            if (val >= 0 && val <= 3) {
                ClariwebAuth.setsessionexpiry(val);
                var next = $scope.transcodeStateName($stateParams.n);
                if (angular.isDefined(next) && next && next !== "login") {
                    $state.go(next, {
                        'query_params': $httpParamSerializer($stateParams.query_params),
                        'document_id': $stateParams.i
                    });
                } else {
                    if (ClariwebAuth.isclinic() && ClariwebCustomSettings.get("CLINIC_ENABLED")) {
                        $state.go("clinic.list");
                    } else {
                        $state.go("document.list");
                    }
                }
            }
        };
        $scope.dateopened = false;
        $scope.minDate = new Date(1900, 1, 1);
        $scope.maxDate = new Date();
        $scope.dateOptions = {
            'starting-day': 1,
            'show-weeks': false,
            'minDate': $scope.minDate,
            'maxDate': $scope.maxDate,
            'minMode': 'day',
            'maxMode': 'day',
            'datepickerMode': 'day'
        };
        $scope.dateopen = function($event) {
            $event.preventDefault();
            $event.stopPropagation();
            $scope.dateopened = true;
        };
        $scope.$watch("birthdate", function(v) {
            var isvalid = v && v > $scope.minDate && v < $scope.maxDate;
            $scope.login_invoice_form.birthdate = {
                '$valid': isvalid,
                '$pristine': !angular.isDefined(v) || v === null
            };
        });
        ClariwebAuth.try_autologin();
    }
)

.controller(
    'PasswordResetCtrl',
    function($scope, $http, $filter, $state, Constants) {
        $scope.reseterrors = function() {
            $scope.passwordmismatch = false;
            $scope.error = '';
            $scope.ratelimited = false;
            $scope.errorlist = [];
            $scope.unavailable = false;
            $scope.previousfail = false;
        };
        $scope.reset = function() {
            $scope.password = '';
            $scope.successphone = '';
            $scope.reseterrors();
            $scope.step = 'ask';
            $scope.password_change_success = false;
        };
        $scope.reset();
        $scope.minDate = new Date(1900, 1, 1);
        $scope.maxDate = new Date();
        $scope.dateOptions = {
            'starting-day': 1,
            'show-weeks': false,
            'minDate': $scope.minDate,
            'maxDate': $scope.maxDate,
            'minMode': 'day',
            'maxMode': 'day',
            'datepickerMode': 'day'
        };
        $scope.open = function($event) {
            $event.preventDefault();
            $event.stopPropagation();
            $scope.opened = true;
        };
        $scope.askpasswordreset = function() {
            $scope.reseterrors();
            $http.post(Constants.API_PASSWORDRESET_ASK, {
                first_name: $scope.first_name,
                last_name: $scope.last_name,
                birthdate: $filter('date')($scope.birthdate, 'yyyy-MM-dd')
            }).then(
                function(response) {
                    $scope.successphone = response.data.mobilephone;
                    $scope.username = response.data.username;
                    $scope.step = 'check';
                },
                function(response) {
                    if (response.status === 404) {
                        $scope.previousfail = true;
                        $scope.error = "Nous ne reconnaissons pas votre identité. Merci de vérifier votre saisie. Peut être vous êtes vous enregistré sous votre nom marital au laboratoire ?";
                    } else if (response.status === 400) {
                        $scope.previousfail = true;
                        $scope.error = "Les données saisies sont invalides.";
                        $scope.errorlist = response.data.errors;
                    } else if (response.status === 406) {
                        $scope.error = "Aucun numéro de téléphone n'est enregistré. La re-initialisation est impossible.";
                    } else if (response.status === 429) {
                        $scope.ratelimited = true;
                    } else if (response.status === 503) {
                        $scope.unavailable = true;
                    } else {
                        $scope.error = "Une erreur inconnue est survenue.";
                    }
                }
            );
        };

        $scope.checkvalidationcode = function() {
            $scope.reseterrors();
            $http.post(Constants.API_PASSWORDRESET_CHECK, {
                username: $scope.username,
                validation_code: $scope.validation_code
            }).then(
                function() {
                    $scope.step = 'edit';
                    $scope.validation_ok = true;
                },
                function(response) {
                    if (response.status === 404) {
                        $scope.error = "Le code de vérification n'est pas correct.";
                    } else if (response.status === 400) {
                        $scope.error = "Les données saisies sont invalides.";
                        $scope.errorlist = response.data.errors;
                    } else if (response.status === 429) {
                        $scope.ratelimited = true;
                    } else {
                        $scope.error = "Une erreur inconnue est survenue.";
                    }
                }
            );
        };

        $scope.editpassword = function() {
            $scope.reseterrors();
            $http.post(Constants.API_PASSWORDRESET_EDIT, {
                username: $scope.username,
                validation_code: $scope.validation_code,
                password: $scope.password
            }).then(
                function() {
                    $scope.password_change_success = true;
                },
                function(response) {
                    if (response.status === 404) {
                        $scope.error = "Le code de vérification n'est pas correct.";
                    } else if (response.status === 400) {
                        $scope.error = "Le mot de passe saisi n'est pas valide.";
                        $scope.errorlist = response.data.errors;
                    } else if (response.status === 429) {
                        $scope.ratelimited = true;
                    } else {
                        $scope.error = "Une erreur inconnue est survenue.";
                    }
                }
            );
        };

        $scope.$watch('step', function(step) {
            $scope.reseterrors();
            if (step === 'ask') {
                $scope.stepnumber = 1;
            } else if (step === 'check') {
                $scope.stepnumber = 2;
            } else if (step === 'edit') {
                $scope.stepnumber = 3;
            }
        });
        $scope.$watch('first_name + last_name + birthdate', function(newvalue, oldvalue) {
            if (newvalue !== oldvalue) { $scope.previousfail = false; }
        });
    }
)

.controller(
    'PasswordCheckCtrl',
    function($scope) {
        $scope.password1 = '';
        $scope.password2 = '';
        $scope.rules = [
            {
                description: 'Contient un chiffre',
                match: function() {
                    return $scope.password1 && $scope.password1.match(/[0-9]/);
                }
            },
            {
                description: 'Contient une majuscule',
                match: function() {
                    return $scope.password1 && $scope.password1.match(/[A-Z]/);
                }
            },
            {
                description: 'Contient une minuscule',
                match: function() {
                    return $scope.password1 && $scope.password1.match(/[a-z]/);
                }
            },
            {
                description: '8 Caractères ou plus',
                match: function() {
                    return $scope.password1 && $scope.password1.length >= 8;
                }
            },
            {
                description: 'Les deux mots de passe sont identiques',
                match: function() {
                    return ($scope.password1 && $scope.password2 && $scope.password1 === $scope.password2);
                }
            }
        ];
        $scope.isvalid = function() {
            var i;
            for(i=0; i < $scope.rules.length; i++) {
                var rule = $scope.rules[i];
                if (! rule.match()) {
                    return false;
                }
            }
            return true;
        };
        $scope.getpassword = function() {
            if ($scope.isvalid()) {
                return $scope.password1;
            } else {
                return '';
            }
        };
        $scope.$watch('getpassword()', function() {
            $scope.$parent.password = $scope.getpassword();
        }, true);
    }
)

.controller(
    'EmailConfirm',
    function($scope, $stateParams, $http, Constants) {
        $scope.ready = false;
        $scope.error = '';
        $scope.expired = false;
        $scope.user = null;
        $scope.emailsent = false;
        if ($stateParams.key) {
            $scope.key = $stateParams.key;
        }
        $scope.sendemail = function() {
            $http.get(Constants.API_EMAIL_CONFIRM_SEND)
                .then(function() {
                    $scope.emailsent = true;
                });
        };
        $scope.$watch('key', function(key) {
            $scope.error = '';
            $scope.expired = false;
            if (key && key.length === 12) {
                $http.post(Constants.API_EMAIL_CONFIRM, {'key': key})
                    .then(
                        function(response) {
                            $scope.user = response.data.user;
                            $scope.ready = true;
                        },
                        function(response) {
                            if (response.status === 404) {
                                $scope.expired = true;
                            }
                            $scope.error = response.data.error;
                            $scope.ready = false;
                        }
                    );
            }
        });
    }
)

.controller(
    'LogoutCtrl',
    function(ClariwebAuth) {
        ClariwebAuth.logout();
    }
)

.controller('SettingsCtrl', function() {})
.controller('ErrorCtrl', function() {})

.controller(
    'ContactCtrl',
    function($scope, ClariwebCustomSettings, $sce) {
        ClariwebCustomSettings.load().then(function() {
            $scope.title = ClariwebCustomSettings.get("CUSTOM_TITLE");
            $scope.email = ClariwebCustomSettings.get("CUSTOM_EMAIL");
            $scope.address = {
                line1: ClariwebCustomSettings.get("CUSTOM_ADDRESS_LINE1"),
                city: ClariwebCustomSettings.get("CUSTOM_ADDRESS_CITY"),
                postalcode: ClariwebCustomSettings.get("CUSTOM_ADDRESS_POSTALCODE")
            };
            var iframe = ClariwebCustomSettings.get("CUSTOM_IFRAME");
            if (iframe) {
                $scope.iframe = $sce.trustAsResourceUrl(iframe);
            }
            $scope.labo = ClariwebCustomSettings.get("CUSTOM_LABO");
            $scope.phone = ClariwebCustomSettings.get("CUSTOM_PHONE");
            var website = ClariwebCustomSettings.get("CUSTOM_WEBSITE");
            if (website) {
                $scope.website = $sce.trustAsResourceUrl(website);
            }
            $scope.onlyiframe = !($scope.address.line1 || $scope.phone || $scope.email);
        });
    }
)


.controller(
    'DebptInfoCtrl',
    function($scope, $filter, $http, $sce, $state, $location,
             ClariwebAuth, Constants) {
        $scope.errors = '';
        $scope.debpt_amount = $filter('currency')(ClariwebAuth.get_debpt() / 100, '');
        $scope.request_payment = function() {
            $scope.errors = '';
            /* Allow 10 minutes session in order to complete payment */
            ClariwebAuth.setsessionexpiry(1);  /* 1 = 10 minutes */
            /* OK, this is a bit weird. By default angularjs does not
             * handle well application installed in a subpath
             * like "http://test/myapp/#/state"
             * so we must get the current absolute url, then remove
             * the state part, and finally append the new state we want to get
             * the true url.
             */
            var current_url = $location.absUrl();
            var return_url = current_url.replace($state.href($state.current, {}), '');
            return_url += $state.href('payment', {status: ''});
            $http.post(Constants.API_PAYMENT_REQUEST, {
                "return_url": return_url
            })
            .then(
                function(response) {
                    $scope.payzen_url = $sce.trustAsResourceUrl(response.data.payzen_url);
                    var postdata = angular.copy(response.data);
                    delete postdata.payzen_url;
                    $scope.paymentrequest = postdata;
                },
                function(response) {
                    /* Disable autologin again */
                    ClariwebAuth.setsessionexpiry(0);
                    $scope.errors = response.data.errors;
                }
            );
        };
    }
)

.controller(
    'PaymentReturnController',
    function($scope, $stateParams, $http, $state, Constants, ClariwebAuth) {
        $scope.status = undefined;
        if (! angular.isObject($stateParams.query_params) ||
                Object.keys($stateParams.query_params).length === 0) {
            /* Strange ? */
            $state.go("paymentlist");
        } else {
            $scope.errors = null;
            $http.post(Constants.API_PAYMENT_RESPONSE,
                $stateParams.query_params)
            .then(
                function(response) {
                    /* Disable autologin */
                    ClariwebAuth.setsessionexpiry(0);
                    if (angular.isDefined(response.data.debpt)) {
                        ClariwebAuth.user.debpt = response.data.debpt;
                        ClariwebAuth.user.debpt_overlimit = false;
                    }
                    $scope.status = response.data.success;
                    $scope.errors = null;
                },
                function(response) {
                    /* Disable autologin */
                    ClariwebAuth.setsessionexpiry(0);
                    $scope.status = "error";
                    $scope.errors = response.data.errors;
                }
            );
        }
    }
)

.controller(
    'PayementRequestListController',
    function($scope, $http, Constants) {
        $scope.inLoad = false;
        $scope.payments = [];
        $scope.currentPage = 1;
        $scope.totalPayments = 0;
        $scope.$watch('currentPage', function() {
            $scope.fetch();
        });
        $scope.onselectpage = function(page) {
            $scope.currentPage = page;
        };
        $scope.fetch = function fetch() {
            if ($scope.inLoad) {
                return;
            }
            $scope.inLoad = true;
            $http({
                method: 'GET',
                url: Constants.API_PAYMENT_REQUEST_LIST,
                params: { page: $scope.currentPage }
            })
            .then(
                function(response) {
                    $scope.payments = response.data.results;
                    $scope.totalPayments = response.data.count;
                    $scope.inLoad = false;
                },
                function() {
                    $scope.inLoad = false;
                }
            );
        };
        $scope.fetch();
    }
)
;
