angular.module('clariweb.directives', [])


.directive(
    'loading',
    function($timeout, ClariwebLoading) {
        return {
            restrict: 'E',
            replace: true,
            transclude: true,
            template: "<span ng-transclude ng-show='showloading'></span>",
            link: function(scope) {
                scope.clariwebloading = ClariwebLoading;
                scope.loadingtimeout = null;
                scope.showloading = false;
                scope.$watch("clariwebloading.loading", function(loading) {
                    if (scope.loadingtimeout) {
                        $timeout.cancel(scope.loadingtimeout);
                    }
                    if (loading) {
                        scope.loadingtimeout = $timeout(function() {
                            scope.showloading = true;
                        }, 250);
                    } else {
                        scope.showloading = false;
                    }
                });
            }
        };
    }
)

/* Trying to "fix" autofill problem with angularjs
 * https://github.com/angular/angular.js/issues/1460#issuecomment-27152413
 */
.directive(
   'autofillsync',
   function($timeout) {
       return {
            require: 'ngModel',
            link: function(scope, elem, attrs, ngModel) {
                var origVal = elem.val();
                $timeout(function () {
                    var newVal = elem.val();
                    if(ngModel.$pristine && origVal !== newVal) {
                        ngModel.$setViewValue(newVal);
                    }
              }, 500);
            }
       };
   }
)

.directive(
    'highcharts',
    function() {
        return {
            restrict: 'E',
            template: '<div></div>',
            scope: {
                chartData: "=value",
                chartObj: "=?"
            },
            transclude: true,
            replace: true,
            link: function($scope, $element, $attrs) {
                $scope.$watch('chartData', function(value) {
                    if (!value) {
                        return;
                    }
                    // use default values if nothing is specified in the given settings
                    $scope.chartData.chart.renderTo = $scope.chartData.chart.renderTo || $element[0];
                    if ($attrs.type) {
                        $scope.chartData.chart.type = $scope.chartData.chart.type || $attrs.type;
                    }
                    if ($attrs.height) {
                        $scope.chartData.chart.height = $scope.chartData.chart.height || $attrs.height;
                    }
                    if ($attrs.width) {
                        $scope.chartData.chart.width = $scope.chartData.chart.width || $attrs.width;
                    }
                    require.ensure([], function() {
                        var Highcharts = require("highcharts");
                        $scope.chartObj = new Highcharts.Chart($scope.chartData);
                    });
                });
            }
        };
    }
)

.directive('themerLink', function() {
    "use strict";
    return {
        restrict: 'E',
        template:  '<link ng-if="selected" rel="stylesheet" ng-href="{{ selected.href }}" />',
        replace: true,
        controller: function($scope, ClariwebThemer) {
            $scope.$on("themer:selectionchanged", function() {
                var selection = ClariwebThemer.getSelected();
                if (selection !== null) {
                    Array.prototype.forEach.call(document.querySelectorAll('link[rel=stylesheet]:not([class="ng-scope"])'), function(element){
                        try {
                            element.parentNode.removeChild(element);
                        } catch(err) {}
                    });
                    $scope.selected = selection;
                }
            });
        }
    };

})

.directive('themerSwitcher', function() {
    "use strict";
    return {
        restrict: 'E',
        template: '<select class="form-control" ng-model="theme.selected"><option ng-repeat="style in theme.styles" value="{{ style.key }}">{{ style.label }}</option></select>',
        scope: {},
        replace: true,
        controller: function($scope, ClariwebThemer) {
            $scope.theme = {
                styles: [],
                selected: null
            };
            $scope.$on('themer:listchanged', function() {
                $scope.theme = {
                    styles: ClariwebThemer.styles,
                    selected: ClariwebThemer.getSelected().key
                };
            });
            $scope.$on('themer:selectionchanged', function() {
                $scope.theme.selected = ClariwebThemer.getSelected().key;
            });
            $scope.$watch('theme.selected', function() {
                if (!$scope.theme.selected) { return; }
                ClariwebThemer.setSelected($scope.theme.selected);
            });
        }
    };
})

.directive('onSizeChanged', function ($window) {
    return {
        restrict: 'A',
        scope: {
            onSizeChanged: '&'
        },
        link: function (scope, $element) {
            var element = $element[0];
            function cacheElementSize(scope, element) {
                scope.cachedElementWidth = element.offsetWidth;
                scope.cachedElementHeight = element.offsetHeight;
            }
            function onWindowResize() {
                var isSizeChanged = scope.cachedElementWidth !== element.offsetWidth || scope.cachedElementHeight !== element.offsetHeight;
                if (isSizeChanged) {
                    var expression = scope.onSizeChanged();
                    expression();
                }
            }
            cacheElementSize(scope, element);
            $window.addEventListener('resize', onWindowResize);
        }
    };
})

;