import * as Sentry from '@sentry/browser';

angular.module('clariweb.services', [])

.factory('Constants', function() {
    return {
        DEFAULT_INACTIVITY_TIMEOUT: 300 * 1000,
        SMS_PRICE: 0.11,
        API: "../api/",
        API_LOGIN: "../api/login",
        API_LOGIN_INVOICE: "../api/login/byinvoice",
        API_LOGIN_SETSESSIONEXPIRY: "../api/login/setsessionexpiry",
        API_LOGOUT: "../api/logout",
        API_EMAIL_CONFIRM: "../api/emailconfirm",
        API_EMAIL_CONFIRM_SEND: "../api/emailconfirm/resend",
        API_DOCUMENT_LISTALL: '../api/document/listall',
        API_DOCUMENT_LISTNOTSEEN: '../api/document/listnotseen',
        API_DOCUMENT_DOWNLOAD: '../api/document/{document_id}/download',
        API_DOCUMENT_DOWNLOADSTREAM: '../api/document/{document_id}/downloadstream/',
        API_DOCUMENT_DETAIL: '../api/document/{document_id}/detail',
        API_CLINIC_DOCUMENT_LIST: '../api/document/clinic/list',
        API_CLINIC_DOCUMENT_LIST_NOSHARE: '../api/document/clinic/list/noshareservice',
        API_CLINIC_DOCUMENT_DETAIL: '../api/document/clinic/{document_id}/detail',
        API_SETTINGS_LOAD: '../api/settings',
        API_PASSWORDRESET_ASK: '../api/passwordreset/ask',
        API_PASSWORDRESET_CHECK: '../api/passwordreset/check',
        API_PASSWORDRESET_EDIT: '../api/passwordreset/edit',
        API_PAYMENT_REQUEST: '../api/payment/request',
        API_PAYMENT_REQUEST_LIST: '../api/payment/request/list',
        API_PAYMENT_RESPONSE: '../api/payment/response',
        API_ADMIN_NOTIFICATION: '../api/admin/notification',
        API_ADMIN_ACCESSLOG: '../api/admin/accesslog',
        API_ADMIN_SMSCOUNT: '../api/admin/smscount',
        API_ADMIN_SMSCOUNT_QUOTA: '../api/admin/smscount/quota',
        API_ADMIN_SMSCOUNT_UPDATEQUOTA: '../api/admin/smscount/quota/update',
        API_ADMIN_BUYSMS: '../api/admin/smscount/buy',
        API_ADMIN_LOGINAS_SELECTOR: '../api/admin/loginas/selector',
        API_ADMIN_LOGINAS: '../api/admin/loginas',
        API_ADMIN_SETTINGS: '../api/admin/settings',
        API_ADMIN_USERMODIFY: '../api/admin/usermodify',
        API_ADMIN_STATS_MONTH_USER: '../api/admin/stats/month/user',
        API_ADMIN_STATS_MONTH_GRAPH: '../api/admin/stats/month',
        API_ADMIN_PAYMENTS: '../api/admin/payments'
    };
})

.factory(
    'ClariwebCustomSettings',
    function(Constants, $http, $state) {
        var Settings = {
            settings: {},
        };
        Settings.load = function() {
            return $http.get(Constants.API_SETTINGS_LOAD, {cache: true})
                .then(
                    function(response) {
                        Settings.settings = response.data;
                    },
                    function() {
                        $state.go("error.serverdown");
                    }
                );
        };
        Settings.get = function(key) {
            return Settings.settings[key];
        };
        return Settings;
    }
)

.factory(
    'ClariwebAuth',
    function(Constants, $http, $cookies, $rootScope, $filter, $q, localStorageService) {
        var Auth = {};

        var setRavenUserData = function(user) {
            Sentry.configureScope((scope) => {
                scope.setUser({
                    "username": user.username,
                    "email": user.email
                });
            });
        };

        Auth.reset = function() {
            Auth.loggedin = false;
            Auth.user = {};
            Auth.sessionid = null;
            Auth.ratelimited = false;
            Auth.logged_as = undefined;
            /* 5 minutes timeout (default) */
            Auth.inactivityTimeout = Constants.DEFAULT_INACTIVITY_TIMEOUT;
        };

        Auth.reset();

        Auth.isstaff = function() {
            return Auth.loggedin && Auth.user && Auth.user.is_staff;
        };

        Auth.ispatient = function() {
            return Auth.loggedin && Auth.user && Auth.user.kind === "PAT";
        };

        Auth.isclinic = function() {
            return Auth.loggedin && Auth.user && Auth.user.kind === "COR";
        };

        Auth.is_session_supported = function() {
            return localStorageService.isSupported;
        };

        Auth.invoice_only = function() {
            return Auth.loggedin && Auth.user && Auth.user.invoice_only;
        };

        Auth._login = function(url, credentials) {
            var q = $http({
                method: 'POST',
                cache: false,
                url: url,
                data: credentials,
                ignoreAuthModule: true
            });
            q.then(
                function(response) {
                    Auth.user = response.data.user;
                    Auth.sessionid = response.data.sessionid;
                    Auth.ratelimited = false;
                    Auth.loggedin = true;
                    Auth.login_using_session();
                },
                function(response) {
                    Auth.reset();
                    Auth.ratelimited = (response.status === 429);
                }
            );
            return q;
        };

        Auth.login = function(username, password) {
            var credentials = {};
            if (username && password) {
                credentials = {username: username, password: password};
            }
            return Auth._login(Constants.API_LOGIN, credentials);
        };

        Auth.login_by_invoice = function(invoice_id, birthdate) {
            var credentials = {};
            if (invoice_id && birthdate) {
                credentials = {
                    invoice_id: invoice_id,
                    birthdate: $filter('date')(birthdate, 'yyyy-MM-dd')
                };
            }
            return Auth._login(Constants.API_LOGIN_INVOICE, credentials);
        };

        Auth.loginas = function(username) {
            var q = $http.post(Constants.API_ADMIN_LOGINAS, {"username": username})
                .then(
                    function(response) {
                        if (response.data.username !== Auth.user.username) {
                            Auth.logged_as = response.data.username;
                        } else {
                            Auth.logged_as = undefined;
                        }
                    },
                    function() {
                        Auth.logged_as = undefined;
                    }
                );
            return q;
        };

        Auth.try_autologin = function() {
            var deferred = $q.defer();

            if (Auth.is_session_supported() && localStorageService.get("sessionid")) {
                $http.defaults.headers.common['X-Session-Id'] = localStorageService.get("sessionid");
                $http({
                    method: 'GET',
                    url: Constants.API_LOGIN,
                    headers: {
                        'X-SESSION-ID': localStorageService.get("sessionid")
                    },
                    data: {'toto': '1'},
                    ignoreAuthModule: true
                }).then((response) => {
                    /* Disable inactivity timeout checker */
                    if (response.data.expiry_time) {
                        Auth.inactivityTimeout = (response.data.expiry_time * 1000) + 1;
                    } else {
                        Auth.inactivityTimeout = Constants.DEFAULT_INACTIVITY_TIMEOUT + 1;
                    }
                    Auth.user = response.data.user;
                    setRavenUserData(Auth.user);
                    Auth.sessionid = response.data.sessionid;
                    Auth.ratelimited = false;
                    Auth.loggedin = true;
                    Auth.login_using_session();
                    deferred.resolve(true);
                }, () => {
                    deferred.resolve(false);
                });
            } else {
                deferred.resolve(false);
            }

            return deferred.promise;
        };

        Auth.logout = function() {
            Auth.reset();
            delete $http.defaults.headers.common['X-Session-Id'];
            delete $cookies.sessionid;  // TODO: remove
            delete $cookies.csrftoken;
            if (Auth.is_session_supported()) {
                localStorageService.clearAll();
            }
            var q = $http.get(Constants.API_LOGOUT)
                .then(function() {
                    Auth.loggedin = false;
                });
            return q;
        };

        Auth.login_using_session = function() {
            var sessionid = Auth.get_sessionid();
            if (sessionid) {
                $http.defaults.headers.common['X-Session-Id'] = sessionid;
            } else {
                delete $http.defaults.headers.common['X-Session-Id'];
            }
        };

        Auth.get_sessionid = function() {
            if (Auth.loggedin && Auth.sessionid !== null) {
                return Auth.sessionid;
            } else {
                return null;
            }
        };

        Auth.get_debpt = function() {
            if (Auth.user && Auth.user.debpt) {
                return Auth.user.debpt;
            } else {
                return 0;
            }
        };

        Auth.is_debpt_overlimit = function() {
            return Auth.user && Auth.user.debpt_overlimit;
        };

        Auth.setsessionexpiry = function(val) {
            if (Auth.is_session_supported()) {
                if (val === 0 && localStorageService.get("sessionid")) {
                    localStorageService.remove('sessionid');
                } else if (val > 0 && val <= 3) {
                    $http.post(Constants.API_LOGIN_SETSESSIONEXPIRY, {
                        'expiry_choice': val
                        }).then(
                        function(response) {
                            localStorageService.set('sessionid', Auth.sessionid);
                            Auth.inactivityTimeout = response.data.expiry_time * 1000;
                        },
                        function() {
                            Auth.inactivityTimeout = Constants.DEFAULT_INACTIVITY_TIMEOUT;
                        }
                    );
                }
            }
        };
        return Auth;
    }
)

.factory(
    'ClariwebLoading',
    function() {
        var data = {"loading": false};
        data.load = function() {
            data.loading = true;
        };
        data.done = function() {
            data.loading = false;
        };
        return data;
    }
)

.factory('ClariwebThemer', function($rootScope) {
    var Themer = {
        styles: [],
        selected: { key: '', label: '', href: '' },
        watchers: []
    };
    Themer.setStyles = function(stylesmanifest) {
        var styles = [];
        for (var k in stylesmanifest) {
            var href = stylesmanifest[k];
            var rmatch = /^theme-([^.]+)\.css$/;
            var match = rmatch.exec(k);
            if (match !== null && match.length > 1) {
                var label = match[1];
                styles.push({
                    'label': label === 'main' ? 'Standard' : label,
                    'href': href,
                    'key': k
                });
            }
        }
        Themer.styles = styles;
        $rootScope.$broadcast('themer:listchanged');
    };
    Themer.getSelected = function() {
        return angular.copy(Themer.selected);
    };
    Themer.setSelected = function(key) {
        angular.forEach(Themer.styles, function (style) {
            Themer.selected = style;
            if (style.key === key) {
                $rootScope.$broadcast('themer:selectionchanged');
                return null;
            }
        });
    };
    return Themer;
})


;